//
// TextUtil.h: header for the TextUtil class.
//

#ifndef TEXTUTIL_H
#define TEXTUTIL_H

#include <algorithm>
#include <iostream>
#include <string>
#include <vector>
#include <fstream>
#include <map>

using std::string; 
using std::vector; 
using std::pair; 
using std::ifstream; 
using std::cerr; 
using std::endl; 
using std::cout; 
using std::cin; 

using namespace std;

typedef map<string, size_t> Map;
typedef Map::const_iterator MapIter;
typedef vector< pair<int, int> > IntPairVector; 
typedef vector< pair<string, int> > StringIntVector;
typedef multimap<size_t, string> MMap;
typedef MMap::const_reverse_iterator MMRI;

typedef vector<string> strings; 
typedef vector<size_t> counters; 
typedef strings::const_iterator strItr;

class TextUtil {

	friend std::ostream& operator<< (std::ostream& os, TextUtil n);

	public: 
		TextUtil( const string& filename );				// constructor
		IntPairVector wordLengths() const;				// 
		size_t wordCount(const string& word) const;		// 
		StringIntVector topNWords(size_t n) const;		// 
		inline strings& getWords() { return m_words; }

	private:

		int		parseTxtFile();							// parse text file
		string	stripPunctuation(const string& word);	// strip punctuation from word
		

		string	m_filename;								// filname being read
		string	m_legal;								// set of legal letters
		strings m_words;								// list of words in file
		Map		m_wordCount;							// map of words and their word count
		MMap	m_wordFreq;								// multimap of word frequencies

		size_t	m_longest;								// size of the longest word

};
#endif // TEXTUTIL_H
