////////////
//
//  intstack.cpp 
//
//      AUTHOR: Russell Lowke
//   CSCIE-215: Assignment One
//		  DATE: Oct 15th 2004
//
//
// Description: implementation for the Intstack class.
//            
//       Notes: 
//

#include <iostream>	
#include "intstack.h"

using namespace std;

IntStack::IntStack(size_t n) {

	if (n < 0) {
		cerr << "IntStack must be initialized with a positive integer." << endl;
		n = 0;
	}
	
	max_size = n;
	
	stack = new int[max_size];
	
	// reset the stack values
	reset();
}

	
// destructor
IntStack::~IntStack() {
	delete []stack;
}


void IntStack::reset() {

	// erase all data in stack
	for (size_t i = 0; i < max_size; ++i) {
		stack[i] = 0;
	}
	
	// reset the top of the stack
	top_of_stack = 0;
}

//
// push value n onto the stack
//
void IntStack::push(int n) {

	// if full then double the capacity
	if (top_of_stack == max_size) {
		doubleCapacity();
	}
	
	// put n on stack
	stack[top_of_stack] = n;
		
	// increment top_of_stack counter
	top_of_stack += 1;
}

//
//  push array a onto the stack
//
void IntStack::push(int a[], size_t n) {

	// iterate through a[], pushing values
	for (size_t i = 0; i < n; ++i) {
		push(a[i]);
	}
}


//
// pop a value off the stack
//
int  IntStack::pop() {		
	
	if (top_of_stack == 0) {
		cerr << "ERR: Can't pop value, stack is empty" << endl;
		return -1;
	} else {
	
		// get the value on the top of the stack
		int top_value = stack[top_of_stack - 1];
		
		// clear top value from stack
		stack[top_of_stack - 1] = 0;
		
		// decrement top_of_stack counter
		top_of_stack -= 1;
		
		// return top value
		return top_value;
	}
}

//
//  pop n items into array a
//
void IntStack::pop(int a[], size_t n) {

	// fill array a[] with popped values
	for (size_t i = 0; i < n; ++i) {
		if (top_of_stack != 0) {
			a[i] = pop();
		}
	}
}

//
//  returns if the stack is full or not
//
bool IntStack::is_full() {
	return (top_of_stack == max_size);
}

//
//  returns if the stack is empty or not
//
bool IntStack::is_empty() {
	return (top_of_stack == 0);
}

//
//  doubles the capacity of IntStack.
//
void IntStack::doubleCapacity() {

	if (max_size == 0) {							// special case for 0 max
		delete []stack;
		max_size = 1;
		stack = new int[max_size];
		return;
	}

	int* dbl_stack = new int[max_size * 2];		    // make a double size stack
	for(size_t i = 0; i < max_size; ++i) {			// copy stack values into it
		dbl_stack[i] = stack[i];
	}

	delete []stack;									// release the stack, then
	stack = dbl_stack;								// reassign to dbl_stack
	max_size *= 2;									// set the new max_size
}

size_t IntStack::size() {
	return top_of_stack;
}

size_t IntStack::capacity() {
	return max_size;
}
		

//
//  print all the items in a stack
//
void printStack(IntStack* p) {
	
	// find the number of items in stack
	size_t n = p->top_of_stack;
	
	// iterate through all items on stack
	for (size_t i = 0; i < n ; ++i) {
		cout << "[" << (unsigned int) i << "] " << p->stack[i] << endl;
	}
}