
//////////////////////////////////////////////////////////////////////
//
// String.h: interface for the String class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(STRING_H_INCLUDED)
#define STRING_H_INCLUDED

#include <iostream>

class String {

	friend std::ostream& operator<< (std::ostream& os, String s);

public:
	String();
	String(const char* s);
	String(const String& s);
	String(char ch, size_t count = 1);
	~String();

	size_t	length() const;							// return the length of String
	String& append(const String& other);			// append String other to String this
	String& append(const char* other);				// append char*
	String  substr( size_t start, 
					size_t numChars) const;			// return new sub string
	char    char_at( size_t pos ) const;			// return character at position pos

	char	operator[](size_t) const;				// mimics behavior of char_at()
	String& operator+=(const String& other);		// mimics behaviour of append
	String& operator= (const String& other);		// handle assignment of Strings
	bool	operator==(const String& other) const;	// test for case-sensitive equality
	bool	operator< (const String& other) const;	// test for case-sensitive ordering


private:
	void init();
	void ensureSize( size_t size );

	char * m_buffer;								// ptr to buffer with char values
	size_t m_length;								// number of char values
	size_t m_bufSize;								// size of the buffer
};

#endif // !defined(STRING_H_INCLUDED)
