//////////////////////////////////////////////////////////////////////
//
// String.cpp: implementation of the String class.
//
//////////////////////////////////////////////////////////////////////

#include <cstdlib>
#include <cstring>
#include "String.h"

// *****************************************
// The following functions were written by Ken
// and were not supposed to be changed

std::ostream& operator<< (std::ostream& os, String s)
{
	s.ensureSize (s.m_length + 1);
	s.m_buffer [s.m_length] = 0;
	os << "S(" << s.m_length << "," << s.m_bufSize << ")[" << s.m_buffer << "]";
	return os;
}

String::String()
{
	init();
}

String::String (const char* s)
{
	init();
	size_t len = strlen(s);
	ensureSize (len);
	memcpy (m_buffer, s, len);
	m_length = len;
}

String::String (const String& s)
{
	init();
	size_t len = s.m_length;
	ensureSize (len);
	memcpy (m_buffer, s.m_buffer, len);
	m_length = len;
}

String::String (char ch, size_t count /* = 1 */)
{
	init();
	ensureSize (count);
	memset (m_buffer, ch, count);
	m_length = count;
}

String::~String()
{
	delete[] m_buffer;
}

void String::init()
{
	m_buffer = NULL;
	m_length = 0;
	m_bufSize = 0;
}

void String::ensureSize (size_t size)
{
	if (size <= m_bufSize)
	{
		return;
	}
	
	char* pNew = new char[size];
	memcpy (pNew, m_buffer, m_length);
	m_bufSize = size;
	delete[] m_buffer;
	m_buffer = pNew;
}

// *****************************************
// The following functions implement the assignment

size_t String::length() const
{
	return m_length;
}

String& String::append (const String& other)
{
	// deal with a self append
	if (this == &other)
	{
		String copy (other);
		return append (copy.m_buffer, copy.m_length);
	}
	
	return append (other.m_buffer, other.m_length);
}

String& String::append (const char* other)
{
	if (other != NULL)
	{
		size_t length = strlen (other);
		return append (other, length);
	}
	
	return *this;
}

// This overload called by the other two for the purposes of
// consolidating code.
String& String::append (const char* other, size_t length)
{
	size_t newLength = m_length + length;
	ensureSize (newLength);
	memcpy (&m_buffer[m_length], other, length);
	m_length = newLength;
	return *this;
}

String String::substr (size_t start, size_t numChars) const
{
	String sub;

	if (start >= m_length)
	{
		std::cerr << "Invalid start index" << std::endl;
	}
	else
	{
		sub.append (m_buffer + start, __min (numChars, m_length - start));
	}

	return sub;
}

char String::char_at (size_t pos) const
{
	if (pos >= m_length)
	{
		std::cerr << "Bad index" << std::endl;
		return '\0';
	}
	
	return m_buffer[pos];
}

char String::operator[] (size_t pos) const
{
	return char_at (pos);
}

String& String::operator+= (const String& other)
{
	return append (other);
}

String& String::operator+= (const char* other)
{
	return append (other);
}

String& String::operator= (const String& other)
{
	if (this != &other)
	{
		return assign (other.m_buffer, other.m_length);
	}
	
	return *this;
}

String& String::operator= (const char* other)
{
	return assign (other, strlen (other));
}

String& String::assign (const char* other, size_t numChars)
{
	ensureSize (numChars);
	memcpy (m_buffer, other, numChars);
	m_length = numChars;
	return *this;
}

bool String::operator== (const String& other) const
{
	return equivalent (other.m_buffer, other.m_length);
}

bool String::operator== (const char* other) const
{
	return equivalent (other, strlen (other));
}

bool String::equivalent (const char* other, size_t numChars) const
{
	if (m_length != numChars)
		return false;
	
	return memcmp (m_buffer, other, m_length) == 0;
}

bool String::operator< (const String& other) const
{
	return less_than (other.m_buffer, other.m_length);
}

bool String::operator< (const char* other) const
{
	return less_than (other, strlen (other));
}

bool String::less_than (const char* other, size_t length) const
{
	size_t len = __min (m_length, length);
	
	for (size_t i = 0; i < len; ++i)
	{
		if (m_buffer[i] < other[i])
			return true;
		
		if (m_buffer[i] > other[i])
			return false;
	}
	
	return m_length < length;
}

