//////////////////////////////////////////////////////////////////////
//
// String.h: interface for the String class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(STRING_H_INCLUDED)
#define STRING_H_INCLUDED

#include <iostream>

class String
{
	friend std::ostream& operator<< (std::ostream& os, String s);
public:
	String();
	String (const char* s);
	String (const String& s);
	explicit String (char ch, size_t count = 1);
	~String();
	size_t length() const;
	String& append (const String& other);
	String& append (const char* other);
	String& append (const char* other, size_t length);
	String substr (size_t start, size_t numChars) const;
	char char_at (size_t pos) const;
	char operator[] (size_t pos) const;
	String& operator+= (const String& other);
	String& operator+= (const char* other);
	String& operator= (const String& other);
	String& operator= (const char* other);
	String& assign (const char* other, size_t numChars);
	bool operator== (const String& other) const;
	bool operator== (const char* other) const;
	bool equivalent (const char* other, size_t numChars) const;
	bool operator< (const String& other) const;
	bool operator< (const char* other) const;
	bool less_than (const char* other, size_t length) const;
private:
	void init();
	void ensureSize (size_t size);
	char* m_buffer;
	size_t m_length;
	size_t m_bufSize;
};

#endif
