////////////
//
//  TextUtil.cpp 
//
//      AUTHOR: Russell Lowke
//   CSCIE-215: Assignment Four
//		  DATE: Dec 1st 2004
//
//
// Description: implementation for the TextUtil class.
//            
//       Notes: 
//

#include "TextUtil.h"


TextUtil::TextUtil( const string& filename ) {		// constructor

	m_filename = filename;
	m_legal    = "ABCDEFGHIJKLMNOPQRSTUVWXZYabcdefghijklmnopqrstuvwxyz";

	parseTxtFile();

}

int TextUtil::parseTxtFile() {

	// class ifstream defines an input stream from a file. 
	ifstream file ( m_filename.c_str() ); 

	// Test to see if file is now open; die otherwise 
	if ( ! file.is_open() ) { 
		cerr << "Failed to open file: " << m_filename << endl; 
		return -1; 
	} 

	// Read through the file one word at a time, 
	// stripping punctuation, storing each word, and the length of the longest word
	string word;
	m_longest = 0;
	while ( file >> word ) { 
		word = stripPunctuation(word);
		if (word.length() > 0) { 
			m_words.push_back(word);		// add to word list
			m_wordCount[word]++;			// add to wordCount

			if (word.length() > m_longest) {
				m_longest = word.length();
			}
		}
	}

	// iterate through the word count map creating a word frequency multi-map
	pair<size_t, string> v;
	for (MapIter mi = m_wordCount.begin(); mi != m_wordCount.end(); ++mi) {
		v.first  = mi->second;
		v.second = mi->first;
		m_wordFreq.insert(v);
	}

	return 0;
}


IntPairVector TextUtil::wordLengths() const {

	IntPairVector pairList;

	// iterate through words, counting word lengths
	counters n(m_longest + 1);
	for( strItr iter = m_words.begin(); iter != m_words.end(); ++iter ) {
		++n[(*iter).length()];
	}
	
	// create pairs
	pair<int, int> value;
	for (size_t i = 0; i < m_longest + 1; ++i) {
		if (n[i] > 0) {
			value.first  = (int) i;
			value.second = (int) n[i];
			pairList.push_back(value);
		}
	}

	return pairList;
}

size_t TextUtil::wordCount(const string& word) const {
	MapIter itr = m_wordCount.find(word);

	if (itr == m_wordCount.end()) {
		return 0;
	} else {
		return m_wordCount.find(word)->second;
	}
	
	// return (size_t) m_wordCount[word];		// can't use due to const
}


StringIntVector TextUtil::topNWords(size_t n) const {

	StringIntVector pairList;
	size_t counter = 0;

	// iterate through word frequency multi-map in reverse order
	// taking the first n values.
	pair<string, int> value;
	for (MMRI mi = m_wordFreq.rbegin(); mi != m_wordFreq.rend(); mi++) {
		if (counter < n) {
			value.first  = mi->second;
			value.second = mi->first;
			pairList.push_back(value);
			++counter;
		}
	}

	return pairList;
}

string TextUtil::stripPunctuation(const string& word) {

	string nWord = "";

	size_t first = word.find_first_of(m_legal);
	size_t last  = word.find_last_of(m_legal);
	
	if ( ! (first == -1 || last == -1)) {
		nWord = word.substr(first, last - first + 1);
	}

	return(nWord);
}


std::ostream& operator<< (std::ostream& os, TextUtil n) {

	strings words = n.getWords();

	// iterate through words and print
	for( strItr iter = words.begin(); iter != words.end(); ++iter ) {
		os << *iter << endl;
	}

	return os;
}