#include "textutil.h"
#include <fstream>
#include <iostream>
#include <algorithm>

using std::string;
using std::cerr;
using std::endl;

void trimPunct (string* word)
{
	word->erase (word->begin(), std::find_if (word->begin(), word->end(), isalpha));
	word->erase (std::find_if (word->rbegin(), word->rend(), isalpha).base(), word->end());
}

TextUtil::TextUtil (const string& filename)
{
	std::ifstream file (filename.c_str()); 
	
	if (!file) 
	{ 
		cerr << "Failed to open file: " << filename << endl; 
	}
	else
	{
		string word;

		while (file >> word) 
		{ 
			trimPunct (&word);

			if (!word.empty())
			{
				++m_wordCounts[word];
			}
		}
	}
}

IntPairVector TextUtil::wordLengths() const
{
	typedef std::map <size_t, size_t> IntIntMap;
	IntIntMap wordLengthMap;

	for (WordCountMap::const_iterator it = m_wordCounts.begin(); it != m_wordCounts.end(); ++it)
	{
		wordLengthMap[it->first.length()] += it->second;
	}

	IntPairVector wordLengths (wordLengthMap.size());
	std::copy (wordLengthMap.begin(), wordLengthMap.end(), wordLengths.begin());
	return wordLengths;
}

size_t TextUtil::wordCount (const string& word) const
{
	WordCountMap::const_iterator match = m_wordCounts.find (word);
	return match == m_wordCounts.end() ? 0 : match->second;
}

bool GreaterOccurrence (const std::pair<string, size_t>& arg1, const std::pair<string, size_t>& arg2)
{
	return arg1.second > arg2.second;
}

StringIntVector TextUtil::topNWords (size_t n) const
{
	StringIntVector wordOccurrences (std::min (n, m_wordCounts.size()));
	std::partial_sort_copy (m_wordCounts.begin(), m_wordCounts.end(), wordOccurrences.begin(), wordOccurrences.end(), GreaterOccurrence);
	return wordOccurrences;
}
