// account.h: interface for the Account class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(ACCOUNT_H_INCLUDED)
#define ACCOUNT_H_INCLUDED

// For stupid M$ compiler
#pragma warning ( disable : 4786 )

#include"market.h"
#include<iostream>
#include<string>
#include<map>

struct AcctErr {};

struct InsuffFunds : public AcctErr {
	InsuffFunds( int bal, int req ) : balance(bal), request(req) {}
	int balance;
	int request;
};

struct InsuffShares : public AcctErr {
	InsuffShares( const Market::Symbol& sym, int num ): name(sym), nShares(num) {}
	Market::Symbol name;
	int nShares;
};

struct UnknownSymbol : public AcctErr {
	UnknownSymbol( const Market::Symbol& sym ): name(sym) {}
	Market::Symbol name;
};

typedef std::string Symbol;

class Account_base {

	public:

		virtual bool sell( const Symbol& s, unsigned int nShares ) = 0;
		virtual bool buy( const Symbol& s, unsigned int nShares ) = 0;
		virtual bool deposit( unsigned int amount ) = 0;
		virtual bool withdraw( unsigned int amount ) = 0;
		virtual void processInterest() = 0;

	protected:
		Account_base();
		virtual ~Account_base();
};

class Account : public Account_base {

  friend std::ostream& operator<< ( std::ostream& os, const Account& acct );

	public:

		Account( const Market& m );
		virtual ~Account();

		bool sell( const Symbol& s, unsigned int nShares );
		bool buy( const Symbol& s, unsigned int nShares );
		bool deposit( unsigned int amount );
		bool withdraw( unsigned int amount );
		void processInterest();

	protected:
		unsigned int m_cashBalance;
		const Market& m_market;

	private:

		typedef std::map <Symbol, int> HoldingMap;
		typedef HoldingMap::const_iterator HoldingIter;
		HoldingMap m_holdings;

};

class GoldAccount : public Account {

	public:
		GoldAccount( const Market& m );
		virtual ~GoldAccount();
		void processInterest();

	private:

};

class PlatinumAccount : public GoldAccount {

	public:
		PlatinumAccount( const Market& m );
		virtual ~PlatinumAccount();
		bool payBack(unsigned int amount);
		void processInterest();				// charge 1.8% on the credit line
		bool buy( const Symbol& s, unsigned int nShares );	// borrow from credit

	private:
		unsigned int m_credit_line;			// money owed

};

#endif // !defined(ACCOUNT_H_INCLUDED)
