//
// Asteroids.cpp: implementation for the Asteroids class.
//
//
//      AUTHOR: Russell Lowke
//
//		  Date: Jan 19th 2005
//


#include "Asteroids.h"

using namespace std;

Asteroids::~Asteroids() {			//  destructor
	delete[] m_iship;
	delete[] m_iship_explode;
	delete[] m_ismall_rock;
	delete[] m_imedium_rock;
	delete[] m_ibig_rock;
	delete[] m_iexplode;
	delete[] m_istar;
	delete[] m_imine;
}
Asteroids::Asteroids(int w, int h, int wX, int wY) : 
	m_animator(M_CHANNELS), m_wWidth(w), m_wHeight(h), m_wX(wX), m_wY(wY) {

	m_displayRect   = fMatrix(-m_wWidth/2, -m_wHeight/2, 
							  m_wWidth/2,  m_wHeight/2);

	m_mouseY        = 0;
	m_mouseX        = 0;

	m_score		    = 0;
	m_ships		    = 0;
	m_nItems        = 0;
	m_level		    = 0;
	m_level_done    = true;
	m_ship_in_at    = 0;		// clock time at which next player ship appears
	m_immune_til	= 0;		// player ship has immunitty for first second or so
	m_titleBanner   = false;
	
	// load disk images into memory
	m_iship			= loadImages( (string) "ship",		   M_NSHIP);
	m_iship_explode	= loadImages( (string) "ship_explode", M_NSHIP_EXLODE);
	m_ismall_rock	= loadImages( (string) "small_rock",   M_NSMALL_ROCK);
	m_imedium_rock	= loadImages( (string) "medium_rock",  M_NMEDIUM_ROCK);
	m_ibig_rock		= loadImages( (string) "big_rock",	   M_NBIG_ROCK);
	m_iexplode		= loadImages( (string) "explode",	   M_NEXPLODE);
	m_istar			= loadImages( (string) "star",		   M_NSTAR);
	m_imine			= loadImages( (string) "mine",		   M_NMINE);
	m_ibullet		= loadImages( (string) "red_bullet",   M_NBULLET);
	m_ibanner		= loadImages( (string) "banner",	   M_NBANNERS, false);

	m_ishipCounter.loadFromFile("Images/ship_counter.tga");
	m_ib_asteroids.loadFromFile("Images/banner/b_asteroids.tga", false);
	m_ib_star.loadFromFile("Images/banner/b_star.tga", false);
	m_ib_lAsteroid.loadFromFile("Images/banner/b_large_asteroid.tga", false);
	m_ib_mAsteroid.loadFromFile("Images/banner/b_medium_asteroid.tga", false);
	m_ib_sAsteroid.loadFromFile("Images/banner/b_small_asteroid.tga", false);
	m_ib_enemy.loadFromFile("Images/banner/b_enemy.tga", false);
	m_ib_mine.loadFromFile("Images/banner/b_mine.tga", false);
	m_ib_bottom.loadFromFile("Images/banner/b_bottom.tga", false);

	// trigger sounds to load them into memory
	::PlaySound("Sounds/enemy_appears.wav",	NULL, SND_ASYNC);
	::PlaySound("Sounds/enemy_fire.wav",	NULL, SND_ASYNC);
	::PlaySound("Sounds/game_over.wav",		NULL, SND_ASYNC);
	::PlaySound("Sounds/get_star.wav",		NULL, SND_ASYNC);
	::PlaySound("Sounds/kill_asteroid.wav",	NULL, SND_ASYNC);
	::PlaySound("Sounds/kill_enemy.wav",	NULL, SND_ASYNC);
	::PlaySound("Sounds/level.wav",			NULL, SND_ASYNC);
	::PlaySound("Sounds/mine_appears.wav",	NULL, SND_ASYNC);
	::PlaySound("Sounds/ship_dead.wav",		NULL, SND_ASYNC);
	::PlaySound("Sounds/shoot.wav",			NULL, SND_ASYNC);
	::PlaySound("Sounds/shot_mine.wav",		NULL, SND_ASYNC);
	::PlaySound("Sounds/shot_star.wav",		NULL, SND_ASYNC);
	::PlaySound("Sounds/extra_ship.wav",	NULL, SND_ASYNC);
	::PlaySound("Sounds/start_1.wav",		NULL, SND_ASYNC);  // "that's affermative"
	::PlaySound("Sounds/start_2.wav",		NULL, SND_ASYNC);  // "rodger"
	::PlaySound("Sounds/start_3.wav",		NULL, SND_ASYNC);  // "weapons hot"
	::PlaySound("Sounds/start_4.wav",		NULL, SND_ASYNC);  // "engaging weapons"
	::PlaySound("Sounds/start_6.wav",		NULL, SND_ASYNC);  // "I gotem"
	::PlaySound("Sounds/start_7.wav",		NULL, SND_ASYNC);  // "rodger that"
	::PlaySound("Sounds/start_8.wav",		NULL, SND_ASYNC);  // "acknowledged"
	::PlaySound("Sounds/start_5.wav",		NULL, SND_ASYNC);  // "I'm on em"

	size_t sprite = M_CHANNELS;				// make counter for sprite channel

	// player ship sprite is always at top of sprite list
	// being drawn last, above other sprites
	m_ship   = --sprite;

	// player bullet sprites are always just before the player
	m_bullet = --sprite;
	for (int i = 0; i < M_MAXBULLETS-1; ++i)
		--sprite;

	// mines are just before bullets
	m_mineOne = --sprite;
	m_mineTwo = --sprite;

	m_cielingSprite = sprite;		// record top of other sprites

	buildSpriteList();

	// create a dummy level for opening screen
	for (int i = 0; i < 5; ++i)
		addAsteroid(); 
	for (int i = 0; i < 15; ++i) {
		fMatrix location( (float) randomInt(m_displayRect.x(), m_displayRect.z()),
						 (float) randomInt(m_displayRect.y(), m_displayRect.w()), 
						  0.0, 0.0);
		addMedAsteroid(location);
	}
	for (int i = 0; i < 20; ++i) {
		fMatrix location( (float) randomInt(m_displayRect.x(), m_displayRect.z()),
						 (float) randomInt(m_displayRect.y(), m_displayRect.w()), 
						  0.0, 0.0);
		addSmAsteroid(location);
	}
	addMine();
}


void Asteroids::buildSpriteList() {
	m_freeSprites = iList();			// rebuild the freeSprites list
	size_t sprite = m_cielingSprite;
	for(; sprite > 1;)					// Note: channel 0 not used
		m_freeSprites.push_back(--sprite);
}

// set the parameters for each level
void Asteroids::levelData() {
	switch (m_level) {
		case 1:
			// new game!
			m_score		  = 0;
			m_ships		  = 5;

			m_nStars	  = 10;
			m_nMines	  = 0;
			m_nAsteroids  = 8;
			m_medAstSplit = 0;
			m_smAstSplit  = 0;
			break;
		case 2:
			m_nStars	  = 12;
			m_nMines	  = 0;
			m_nAsteroids  = 3;
			m_medAstSplit = 2;
			m_smAstSplit  = 0;
			break;
		case 3:
			m_nStars	  = 14;
			m_nMines	  = 1;
			m_nAsteroids  = 4;
			m_medAstSplit = 2;
			m_smAstSplit  = 0;
			break;
		case 4:
			m_nStars	  = 16;
			m_nMines	  = 1;
			m_nAsteroids  = 3;
			m_medAstSplit = 2;
			m_smAstSplit  = 3;
			break;
		case 5:
			m_nStars	  = 18;
			m_nMines	  = 2;
			m_nAsteroids  = 4;
			m_medAstSplit = 3;
			m_smAstSplit  = 3;
			break;
		default:			// level 6 and higher
			m_nStars	  = 20;
			m_nMines	  = 1;
			m_nAsteroids  = 5;
			m_medAstSplit = 3;
			m_smAstSplit  = 4;
			break;
	}
}

void Asteroids::startLevel() {

	// clear all sprites in the animator
	m_animator.clear();
	buildSpriteList();
	m_level_done = false;
	m_nItems     = 0;

	// build level
	// crystals are added first as they must always
	// draw underneath asteroids
	for (int i = 0; i < m_nStars; ++i)	   { addStar(); }
	for (int i = 0; i < m_nMines; ++i)	   { addMine(); }
	for (int i = 0; i < m_nAsteroids; ++i) { addAsteroid(); }
	addShip();
}


//
// on_display
//
// Window-repainting callback.
//
void Asteroids::on_display() {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// check if need to add player ship
	if (m_ship_in_at && clock() > m_ship_in_at) {
		m_ship_in_at = 0;
		addShip();
	}

	// check if level clear
	if (! m_nItems) { 
		m_animator.setActive(m_ship, false);	// switch off the ship
		m_level_done = true;					// level is done
		++m_level;								// set to next level
		if (m_level > 6) { m_level = 6; }		// limit to lvl 6
		levelData();							// get level data
		m_nItems = -1;							// flag items as done
		sound(LEVEL);
	}

	shipMovement();								// update player ship
	m_animator.update();						// update other sprites
	drawScore();								// draw the score
	if (m_level_done) { drawBanners(); }		// display any level banners
    glutSwapBuffers();							// Swap our back and front buffers
	glutPostRedisplay();						// always have more to draw!
}

void Asteroids::on_keyboard(unsigned char key, int x, int y) {

	switch (key) {

		case '1': case 'a': addAsteroid(); break;
		case '2': case 'm': addMine(); break;
		case '3': case 's': addStar(); break;
		case ' ': 
			if (m_ships < 8) {
				++m_ships;			// cheat with extra ship
				sound(EXTRA_SHIP);
			}
			break;
		default:
			break;
	}

	glutPostRedisplay();
}

//
// on_window_reshape
//
// Window-resizing callback.
//
void Asteroids::on_window_reshape (int w, int h) {
	
	m_wWidth  = w;
	m_wHeight = h;

	glViewport(0, 0, (GLsizei) m_wWidth, (GLsizei) m_wHeight);
    gluOrtho2D (-m_wWidth/2, m_wWidth/2, -m_wHeight/2, m_wHeight/2);
}

void Asteroids::mouseLocation(int x, int y) {
	m_mouseX = x - m_wWidth/2;
	m_mouseY = (m_wHeight - y) - m_wHeight/2;		// invert y for screen-centric frame
}
void Asteroids::on_mouse_move(int x, int y) {
	mouseLocation(x, y);
}
void Asteroids::on_mouse(int button, int state, int x, int y) {	

	if (state != GLUT_DOWN) { return; }
	if (button == GLUT_LEFT_BUTTON) {
		if (m_level_done && ! m_level) { m_level = 1; m_titleBanner  = true; levelData(); }	
		else if (m_level_done) { startLevel(); }	// start a new game
		else
			addBullet();
	}
	if (button == GLUT_RIGHT_BUTTON) { }

	glutPostRedisplay();
}

void Asteroids::drawScore() {

	string dTxt;

	// show the current level
	if (m_level > 0) {
		dTxt = "Level: ";
		char buffer[33];
		itoa (m_level, buffer, 10);
		dTxt += buffer;
		dTxt += "       ";
	}

	// show the last or current score
	dTxt += "Score: ";
	char buffer[33];
	itoa (m_score, buffer, 10);
	dTxt += buffer;

	// displacement of text from btm lft corner
	int disp = 15;		
	// convert score to char and print
	size_t length = dTxt.length();
	glColor3f(1.0, 1.0, 1.0);		// make white
	glRasterPos2f(-m_wWidth/2 + disp, -m_wHeight/2 + disp);
	for (size_t i = 0; i < length; ++i)
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, dTxt[i]);

	// display ship counters
	int x;
	int y = -m_wHeight/2 + disp - 4;
	for (int i = 0; i < m_ships-1; ++i) {
		x = -m_wWidth/2 + 150 + i*(m_ishipCounter.getWidth() + 10);
		m_ishipCounter.drawAt(x, y);
	}
}

void Asteroids::drawBanners() {

	// get placement for banner
	float xmodif  = m_ibanner[m_level].getWidth()/2;
	float ymodif  = (m_wHeight/2) /3;
	float bHeight = m_ibanner[m_level].getHeight();

	if (! m_titleBanner)
		m_ib_asteroids.drawAt(-xmodif , +ymodif);		// opening "Asteroids" banner
	else
		m_ibanner[m_level].drawAt(-xmodif , +ymodif);	// normal banners (0 = "Game Over")

	if (m_level > 0) {
		if (m_nStars) {
			ymodif -= bHeight;
			m_ib_star.drawAt(-xmodif , +ymodif);
		}
		if (m_nAsteroids) {
			ymodif -= bHeight;
			m_ib_lAsteroid.drawAt(-xmodif , +ymodif);
		}
		if (m_medAstSplit) {
			ymodif -= bHeight;
			m_ib_mAsteroid.drawAt(-xmodif , +ymodif);
		}
		if (m_smAstSplit) {
			ymodif -= bHeight;
			m_ib_sAsteroid.drawAt(-xmodif , +ymodif);
		}
		if (m_nMines) {
			ymodif -= bHeight;
			m_ib_mine.drawAt(-xmodif , +ymodif);
		}
		
		// add bottom of banner
		ymodif -= m_ib_bottom.getHeight();
		m_ib_bottom.drawAt(-xmodif , +ymodif);
	}
}

// load a series of images into memory (Note: must delete[] later)
Image* Asteroids::loadImages(string& imageName, size_t n, bool alpha) {

	Image* images = new Image[n];

	for (size_t i = 0; i < n; ++i) {
		stringstream ss;			// initialize/clear the ss
		ss << "Images/" << imageName << "/" << (int) i << "_" << imageName << ".tga"; 
		images[i].loadFromFile(ss.str().c_str(), alpha);
	}

	return images;
}

size_t Asteroids::takeASprite() {
	if (m_freeSprites.size() == 0)
		return 0;						// no sprite channels left!
	size_t n = m_freeSprites.back();
	m_freeSprites.pop_back();
	return n;
}

void Asteroids::returnASprite(size_t n) {
	m_animator.clear(n);
	m_freeSprites.push_back(n);
}

void Asteroids::addShip() {
	m_animator.setLabel(m_ship, NORMAL);
	m_animator.setLocation(m_ship, fMatrix(m_mouseX, m_mouseY, 0.0));
	m_animator.setVelocity(m_ship, fMatrix(0.0, 0.0, 0.0));
	m_animator.setMaxVelocity(m_ship, 2.5);
	m_animator.setWallType(m_ship, W_NONE);
	m_animator.setDisplayRect(m_ship, m_displayRect);
	m_animator.setImage(m_ship, m_iship, M_NSHIP);
	m_animator.setCycleType(m_ship, C_FACING);
	m_animator.setRadius(m_ship, 16);
	m_animator.setActive(m_ship, true);

	// ship gets 1 second of immunity
	m_immune_til = clock() + (clock_t) (1.5 * CLK_TCK);
	sound(START);
}

void Asteroids::addBullet() {

	// can't fire if exploding.
	if (m_animator.getLabel(m_ship) != NORMAL)
		return;

	// get a bullet channel, if any
	size_t sprite = 0;
	for (int i = 0; i < M_MAXBULLETS && ! sprite; ++i)
		if ( ! m_animator.getActive(m_bullet - i))
			sprite = m_bullet - i;

	// determine bullet x & y velocity
	float rise = m_animator.getVelocity(m_ship).y();
	float run  = m_animator.getVelocity(m_ship).x();
	fMatrix velocity = getVelocity_Vel(rise, run, 7.0);

	// ensure no bullets with 0 velocity
	if (velocity.x() == 0 && velocity.y() == 0)
		sprite = 0;

	if (sprite) {
		m_animator.setLocation(sprite, m_animator.getLocation(m_ship));
		m_animator.setVelocity(sprite, velocity);
		m_animator.setWallType(sprite, W_DEACTIVATE);
		m_animator.setDisplayRect(sprite, m_displayRect);
		m_animator.setImage(sprite, m_ibullet, M_NBULLET);
		m_animator.setCycleType(sprite, C_NONE);
		m_animator.setRadius(sprite, 2);
		m_animator.setActive(sprite, true);

		sound(SHOOT);
	}
}

// add a star to the playing field
void Asteroids::addStar() {
	
	// if sprite == 0 then there are no
	// channels left and star cannot be added
	if (size_t sprite = takeASprite()) {
		m_nItems += 1;

		float wMrg = m_wWidth  * 0.05;		// edge margin 5%
		float hMrg = m_wHeight * 0.05;		// edge margin 5%

		fMatrix location( (float) randomInt(m_displayRect.x() + wMrg, m_displayRect.z() - wMrg),
						 (float) randomInt(m_displayRect.y() + hMrg, m_displayRect.w() - hMrg), 
						  0.0);

		fMatrix velocity(0.0, 0.0, 0.0 );

		m_animator.setLabel(sprite, NORMAL);
		m_animator.setLocation(sprite, location);
		m_animator.setVelocity(sprite, velocity);
		m_animator.setWallType(sprite, W_WRAP);
		m_animator.setDisplayRect(sprite, m_displayRect);
		m_animator.setImage(sprite, m_istar, M_NSTAR);
		m_animator.setCycleType(sprite, C_REVERSE);
		m_animator.setCycleFPS(sprite, randomInt(7, 15));
		m_animator.setCollision(sprite, m_ship, STAR_V_SHIP);
		for (size_t i = 0; i < M_MAXBULLETS; ++i)
			m_animator.setCollision(sprite, m_bullet-i, STAR_V_BULLET);
		m_animator.setCollision(sprite, m_mineOne, STAR_V_MINE);
		m_animator.setCollision(sprite, m_mineTwo, STAR_V_MINE);
		m_animator.setRadius(sprite, 13);
		m_animator.setActive(sprite, true);
	}
}

void Asteroids::addMine() {

	// add sprike to one of two sprite channels
	size_t sprite = 0;
	if ( ! m_animator.getActive(m_mineOne))
		sprite = m_mineOne;
	else if ( ! m_animator.getActive(m_mineTwo))
		sprite = m_mineTwo;

	if (sprite) {

		// edge margin 5%
		fMatrix edge(+m_wWidth*0.03, +m_wHeight*0.03, -m_wWidth*0.03, -m_wHeight*0.03);

		fMatrix location( (float) randomInt(m_displayRect.x(), m_displayRect.z()),
						(float) randomInt(m_displayRect.y(), m_displayRect.w()), 
						0.0);
		float speedX = ( (float) randomInt(8, 18) / 10.0);
		if ( randomInt(0, 1) == 1) { speedX = -speedX; }

		float speedY = ( (float) randomInt(8, 18) / 10.0);
		if ( randomInt(0, 1) == 1) { speedY = -speedY; }

		fMatrix velocity(speedX, speedY, 0.0 );

		m_animator.setLabel(sprite, NORMAL);
		m_animator.setLocation(sprite, location);
		m_animator.setVelocity(sprite, velocity);
		m_animator.setWallType(sprite, W_REFLECT);
		m_animator.setDisplayRect(sprite, m_displayRect);  // + edge ???
		m_animator.setImage(sprite, m_imine, M_NMINE);
		m_animator.setCycleFPS(sprite, randomInt(2, 30));	// random spin speed
		if ( randomInt(0, 1) == 1)							// spin half the asteroids the other way
			m_animator.setCycleDirection(sprite, -1); 
		m_animator.setCollision(sprite, m_ship, MINE_V_SHIP);
		for (size_t i = 0; i < M_MAXBULLETS; ++i)
			m_animator.setCollision(sprite, m_bullet-i, MINE_V_BULLET);
		if (sprite == m_mineTwo)
			m_animator.setCollision(sprite, m_mineOne, MINE_V_MINE);
		m_animator.setRadius(sprite, 16);
		m_animator.setActive(sprite, true);
	}
}

// add an asteroid to the playing field
void Asteroids::addAsteroid() {
	
	// if sprite == 0 then there are no
	// channels left and asteroid cannot be added
	if (size_t sprite = takeASprite()) {
		m_nItems += 1;

		fMatrix location( (float) randomInt(m_displayRect.x(), m_displayRect.z()),
						  (float) randomInt(m_displayRect.y(), m_displayRect.w()), 
						   0.0);
		float speedX = (float) (randomInt(1, 6 + m_level) / 10.0);
		if ( randomInt(0, 1) == 1) { speedX = -speedX; }

		float speedY = (float) (randomInt(1, 6 + m_level) / 10.0);
		if ( randomInt(0, 1) == 1) { speedY = -speedY; }

		fMatrix velocity(speedX, speedY, 0.0 );

		m_animator.setLabel(sprite, LARGE);
		m_animator.setLocation(sprite, location);
		m_animator.setVelocity(sprite, velocity);
		m_animator.setWallType(sprite, W_WRAP);
		m_animator.setDisplayRect(sprite, m_displayRect);
		m_animator.setImage(sprite, m_ibig_rock, M_NBIG_ROCK);
		m_animator.setCycleFPS(sprite, randomInt(2, 55));	// random spin speed
		if ( randomInt(0, 1) == 1)							// spin half the asteroids the other way
			m_animator.setCycleDirection(sprite, -1); 
		m_animator.setCycleType(sprite, C_WRAP);
		m_animator.setCollision(sprite, m_ship, ASTEROID_V_SHIP);
		for (size_t i = 0; i < M_MAXBULLETS; ++i)
			m_animator.setCollision(sprite, m_bullet-i, ASTEROID_V_BULLET);
		m_animator.setRadius(sprite, 16);
		m_animator.setActive(sprite, true);
	}
}


// add an asteroid to the playing field
void Asteroids::addMedAsteroid(fMatrix location) {
	
	// if sprite == 0 then there are no
	// channels left and asteroid cannot be added
	if (size_t sprite = takeASprite()) {
		m_nItems += 1;

		float speedX = (float) (randomInt(1, 10) / 10.0);
		if ( randomInt(0, 1) == 1) { speedX = -speedX; }

		float speedY = (float) (randomInt(1, 10) / 10.0);
		if ( randomInt(0, 1) == 1) { speedY = -speedY; }

		fMatrix velocity(speedX, speedY, 0.0 );

		m_animator.setLabel(sprite, MEDIUM);
		m_animator.setLocation(sprite, location);
		m_animator.setVelocity(sprite, velocity);
		m_animator.setWallType(sprite, W_WRAP);
		m_animator.setDisplayRect(sprite, m_displayRect);
		m_animator.setImage(sprite, m_imedium_rock, M_NMEDIUM_ROCK);
		m_animator.setCycleFPS(sprite, randomInt(2, 55));	// random spin speed
		if ( randomInt(0, 1) == 1)							// spin half the asteroids the other way
			m_animator.setCycleDirection(sprite, -1); 
		m_animator.setCycleType(sprite, C_WRAP);
		m_animator.setCollision(sprite, m_ship, ASTEROID_V_SHIP);
		for (size_t i = 0; i < M_MAXBULLETS; ++i)
			m_animator.setCollision(sprite, m_bullet-i, ASTEROID_V_BULLET);
		m_animator.setRadius(sprite, 11);
		m_animator.setActive(sprite, true);
	}
}

void Asteroids::addSmAsteroid(fMatrix location) {

	// if sprite == 0 then there are no
	// channels left and asteroid cannot be added
	if (size_t sprite = takeASprite()) {
		m_nItems += 1;

		float speedX = (float) (randomInt(1, 14) / 10.0);
		if ( randomInt(0, 1) == 1) { speedX = -speedX; }

		float speedY = (float) (randomInt(1, 14) / 10.0);
		if ( randomInt(0, 1) == 1) { speedY = -speedY; }

		fMatrix velocity(speedX, speedY, 0.0 );

		m_animator.setLabel(sprite, SMALL);
		m_animator.setLocation(sprite, location);
		m_animator.setVelocity(sprite, velocity);
		m_animator.setWallType(sprite, W_WRAP);
		m_animator.setDisplayRect(sprite, m_displayRect);
		m_animator.setImage(sprite, m_ismall_rock, M_NSMALL_ROCK);
		m_animator.setCycleFPS(sprite, randomInt(10, 60));	// random spin speed
		if ( randomInt(0, 1) == 1)							// spin half the asteroids the other way
			m_animator.setCycleDirection(sprite, -1);
		m_animator.setCycleType(sprite, C_WRAP);
		m_animator.setCollision(sprite, m_ship, ASTEROID_V_SHIP);
		for (size_t i = 0; i < M_MAXBULLETS; ++i)
			m_animator.setCollision(sprite, m_bullet-i, ASTEROID_V_BULLET);
		m_animator.setRadius(sprite, 8);
		m_animator.setActive(sprite, true);
	}
}


void Asteroids::shipMovement() {

	if (m_animator.getLabel(m_ship) == NORMAL) {

		float basicVelocity, dif, direction;
		float modifier = 28.0;

		fMatrix location = m_animator.getLocation(m_ship);
		fMatrix velocity(0.0, 0.0, 0.0);

		// handle ship movement
		dif = m_mouseY - location.y();
		if (dif > 0)
			direction = +1.0;
		else
			direction = -1.0;

		dif = abs(dif);
		basicVelocity = dif / modifier;
		velocity.y(basicVelocity * direction);

		dif = m_mouseX - location.x();
		if (dif > 0)
			direction = +1;
		else
			direction = -1;

		dif = abs(dif);
		basicVelocity = dif / modifier;
		velocity.x(basicVelocity * direction);

		m_animator.setVelocity(m_ship, velocity);

	}
}


void Asteroids::gameOver() {
	// setting level to zero signifies end of the game
	m_level		 = 0;
	m_level_done = true;
	sound(GAME_OVER);
}

void Asteroids::sound(Sound s) {

	// ensure that ship exploding sound is played
	// in preference to all others.

	switch (s) {	// these sounds should always play uninterrupted
		case SHIP_DEAD:
			::PlaySound("Sounds/ship_dead.wav", NULL, SND_ASYNC);
			break;
		case START:
			switch (randomInt(1, 8)) {		// random start sound
				case 1: ::PlaySound("Sounds/start_1.wav", NULL, SND_ASYNC); break;
				case 2: ::PlaySound("Sounds/start_2.wav", NULL, SND_ASYNC); break;
				case 3: ::PlaySound("Sounds/start_3.wav", NULL, SND_ASYNC); break;
				case 4: ::PlaySound("Sounds/start_4.wav", NULL, SND_ASYNC); break;
				case 5: ::PlaySound("Sounds/start_5.wav", NULL, SND_ASYNC); break;
				case 6: ::PlaySound("Sounds/start_6.wav", NULL, SND_ASYNC); break;
				case 7: ::PlaySound("Sounds/start_7.wav", NULL, SND_ASYNC); break;
				case 8: ::PlaySound("Sounds/start_8.wav", NULL, SND_ASYNC); break;
			}
			break;
		case GAME_OVER:
			::PlaySound("Sounds/game_over.wav", NULL, SND_ASYNC);
			break;
	}

	if (m_animator.getLabel(m_ship) != EXPLODE) {
		switch (s) {
			case SHOOT:
				if (clock() > m_immune_til)
					::PlaySound("Sounds/shoot.wav", NULL, SND_ASYNC);
				break;
			case KILL_ASTEROID:
				::PlaySound("Sounds/kill_asteroid.wav", NULL, SND_ASYNC);
				break;
			case GET_GOODIE:
				if (clock() > m_immune_til)
					::PlaySound("Sounds/get_star.wav", NULL, SND_ASYNC);
				break;
			case SHOT_GOODIE:
				::PlaySound("Sounds/shot_star.wav", NULL, SND_ASYNC);
				break;
			case ENEMY_FIRE:
				::PlaySound("Sounds/enemy_fire.wav", NULL, SND_ASYNC);
				break;
			case KILL_ENEMY:
				::PlaySound("Sounds/kill_enemy.wav", NULL, SND_ASYNC);
				break;
			case GET_STAR:
				::PlaySound("Sounds/get_star.wav", NULL, SND_ASYNC);
				break;
			case SHOOT_MINE:
				::PlaySound("Sounds/shot_mine.wav", NULL, SND_ASYNC);
				break;
			case EXTRA_SHIP:
				::PlaySound("Sounds/extra_ship.wav", NULL, SND_ASYNC);
				break;
			case LEVEL:
				::PlaySound("Sounds/level.wav", NULL, SND_ASYNC);
				break;
		}
	}
}

void Asteroids::gotStar(size_t n) {
	if (m_animator.getLabel(n) == NORMAL) {
		m_nItems -= 1;
		sound(GET_GOODIE);
		m_score += 20;
		m_animator.setActive(n, false);
	}
}

void Asteroids::shotStar(size_t n) {
	if (m_animator.getLabel(n) != EXPLODE) {
		sound(SHOT_GOODIE);		// star shot by player
		m_nItems -= 1;
		m_score -= 10;
		m_animator.setLabel(n, EXPLODE);
		m_animator.setCycleCell(n, 0);
		m_animator.setCycleDirection(n, +1);
		m_animator.setCycleFPS(n, 30);
		m_animator.setCycleType(n, C_DEACTIVATE);
		m_animator.clearCollisions(n);
	}
}
void Asteroids::shotMine(size_t source, size_t target) {

	if (m_animator.getLabel(source) == NORMAL) {
		sound(SHOOT_MINE);
		m_score += 1;
		fMatrix bulletV  = m_animator.getVelocity(target);
		fMatrix bumpV    = getVelocity_Vel(bulletV.y(), bulletV.x(), 0.5);
		fMatrix location = m_animator.getLocation(source);
		fMatrix velocity = m_animator.getVelocity(source);
		location += bulletV;
		velocity += bumpV;
		m_animator.setLocation(source, location);
		m_animator.setVelocity(source, velocity);
	}
}

void Asteroids::killMine(size_t n) {
	if (m_animator.getLabel(n) != EXPLODE) {
		sound(KILL_ASTEROID);
		m_animator.setLabel(n, EXPLODE);
		m_animator.setImage(n, m_iship_explode, M_NSHIP_EXLODE);
		m_animator.setCycleFPS(n, 60);
		m_animator.setCycleType(n, C_DEACTIVATE);
	}
}

void Asteroids::killShip() {
	if (m_animator.getLabel(m_ship) != EXPLODE) {
		sound(SHIP_DEAD);
		m_animator.setLabel(m_ship, EXPLODE);
		m_animator.setImage(m_ship, m_iship_explode, M_NSHIP_EXLODE);
		m_animator.setCycleFPS(m_ship, 60);
		m_animator.setCycleType(m_ship, C_DEACTIVATE);
		m_ships -= 1;
		if (m_ships == 0)
			gameOver();
		else {
			// wait 2.5 seconds before next ship
			m_ship_in_at = clock() + (clock_t) (2.5 * CLK_TCK);
		}
	}
}


void Asteroids::killBullet(size_t n) {
	// deactivate the bullet sprite
	m_animator.setActive(n, false);
}

void Asteroids::killStar(size_t n) {		// star rolled by mine
	if (m_animator.getLabel(n) != EXPLODE) {
		sound(KILL_ASTEROID);
		m_score  -= 10;
		m_nItems -= 1;
		m_animator.setLabel(n, EXPLODE);
		m_animator.setImage(n, m_iexplode, M_NEXPLODE);
		m_animator.setCycleFPS(n, 60);
		m_animator.setCycleType(n, C_DEACTIVATE);
		m_animator.setWallType(n, W_NONE);
		m_animator.clearCollisions(n);
	}
}

void Asteroids::killAsteroid(size_t n) {
	fMatrix location = m_animator.getLocation(n);	// asteroid location
	Label lbl = m_animator.getLabel(n);				// asteroid label

	switch (lbl) {
		case LARGE:
			sound(KILL_ASTEROID);
			m_nItems -= 1;
			m_score += 10;
			if (m_medAstSplit == 0)
				lbl = SMALL;						// explode like a small asteroid
			else {
				if (m_level > 4 && randomInt(1, 10) == 10)
					for (int i = 0; i < 20; ++i)
						addSmAsteroid(location);	// surprise asteroid
				else
					for (int i = 0; i < m_medAstSplit; ++i)
						addMedAsteroid(location);	// split into medium asteroids

				m_animator.setActive(n, false);
			}

			break;
		case MEDIUM:
			sound(KILL_ASTEROID);
			m_nItems -= 1;
			m_score += 15;
			if (m_smAstSplit == 0)
				lbl = SMALL;						// explode like a small asteroid
			else {
				for (int i = 0; i < m_smAstSplit; ++i)
					addSmAsteroid(location);		// split into small asteroids
				m_animator.setActive(n, false);
			}
			break;
		case SMALL:
			sound(KILL_ASTEROID);
			m_nItems -= 1;
			m_score += 20;
			break;
		case EXPLODE:
			// asteroid already in process of exploding
			break;
	}

	if (lbl == SMALL) {			// bang.
		m_animator.setLabel(n, EXPLODE);
		m_animator.setImage(n, m_iexplode, M_NEXPLODE);
		m_animator.setCycleFPS(n, 60);
		m_animator.setCycleType(n, C_DEACTIVATE);
		m_animator.setWallType(n, W_NONE);
		m_animator.clearCollisions(n);
	}
}


// external Animator calls
void Asteroids::collision(size_t source, size_t target, Collision c) {

	switch (c) {
		case STAR_V_SHIP:
			gotStar(source);
			break;
		case STAR_V_MINE:
			killStar(source);
			break;
		case STAR_V_BULLET:
			shotStar(source);
			killBullet(target);
			break;
		case MINE_V_MINE:
			killMine(source);
			killMine(target);
			break;
		case MINE_V_SHIP:
			if (clock() > m_immune_til) {
				if (m_animator.getLabel(m_ship) != EXPLODE)
					killMine(source);
				killShip();
			}
			break;
		case MINE_V_BULLET:
			shotMine(source, target);
			killBullet(target);
			break;
		case ASTEROID_V_BULLET:
			killAsteroid(source);
			killBullet(target);
			break;
		case ASTEROID_V_SHIP:
			if (clock() > m_immune_til) {
				killAsteroid(source);
				killShip();
			}
			break;
	}
}

// called automatically when sprite is deactivated by animator
void Asteroids::deactivated(size_t n) {
	if (n < m_cielingSprite)
		returnASprite(n);
}