//
// Asteroids.h: header for the Asteroids class.
//
//      AUTHOR: Russell Lowke
//
//		  Date: Jan 20th 2005
//

#ifndef Asteroids_H
#define Asteroids_H

#include "Matrix.h"
#include "utils.h"
#include "Animator.h"
#include <vector>
#include <iostream>
#include <ctime>
#include <string>
#include <sstream>
#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include <windows.h>
#include <mmsystem.h>

typedef vector<size_t> iList;

using namespace std;

class Asteroids {

	public:
	   ~Asteroids();							// destructor
		Asteroids(int wWidth,					// build at wX, wY with height and width
			 int wHeight, 
			 int wX, 
			 int wY);

		// getters and setters
		inline int getWindH() const { return m_wHeight; }

		void on_mouse_move(int x, int y);
		void on_display();
		void on_keyboard(unsigned char key, int x, int y);
		void on_window_reshape(int w, int h);
		void on_mouse (int button, int state, int x, int y);

		// external Animator calls
		void collision(size_t source, size_t target, Collision c);
		void deactivated(size_t n);

	private:

		int m_wWidth;							// window width
		int m_wHeight;							// window height
		int m_wX;								// window X loc
		int m_wY;								// window Y loc
		fMatrix m_displayRect;					// 2d rect of display area

		const static size_t M_CHANNELS = 200;
		Animator m_animator;
		iList m_freeSprites;					// list of available sprites for use

		int m_mouseX;
		int m_mouseY;
		int m_delay;							// interval between redraws (milliseconds)

		int  m_level;							// player level
		int  m_score;							// player's score
		int  m_ships;							// number of ships left
		int  m_nItems;							// number of asteroid like items left.
		bool m_level_done;						// false if game in progress
		bool m_titleBanner;						// true if the title banner has been drawn
		clock_t m_ship_in_at;
		clock_t m_immune_til;					// ship has immunity for the first second or so

		// level specific members
		int m_nStars;							// number of stars on this level
		int m_nMines;							// number of mines on this level (1 or 2)
		int m_nAsteroids;						// number of asteroids starting
		int m_medAstSplit;						// number of medium asteroid split from a large
		int m_smAstSplit;						// number of small asteroids split from a medium

		const static size_t M_NSHIP		    = 48;	// number of ship images
		const static size_t M_NSHIP_EXLODE  = 19;	// number of ship_explode images
		const static size_t M_NSMALL_ROCK   = 40;	// number of small_rock images
		const static size_t M_NMEDIUM_ROCK  = 40;	// number of medium_rock images
		const static size_t M_NBIG_ROCK     = 60;	// number of big_rock images
		const static size_t M_NEXPLODE      = 10;	// number of explode images
		const static size_t M_NSTAR		    = 6;	// number of star images
		const static size_t M_NMINE			= 40;	// number of mine images
		const static size_t M_NBULLET		= 1;	// number of bullet images
		const static size_t M_NBANNERS		= 7;	// number of banners, 0 = "Game Over"

		const static size_t M_MAXBULLETS	= 6;	// number of bullet sprites at any one time

		Image* m_iship;								// array  of ship images
		Image* m_iship_explode;						// array  of ship_explode images
		Image* m_ismall_rock;						// array  of small_rock images
		Image* m_imedium_rock;						// array  of medium_rock images
		Image* m_ibig_rock;							// array  of big_rock images
		Image* m_iexplode;							// array  of explode images
		Image* m_istar;								// array  of star images
		Image* m_imine;								// array  of mine images
		Image* m_ibullet;							// array  of bullet images
		Image* m_ibanner;							// array  of level banners, 0 = "Game Over"

		Image  m_ishipCounter;						// small image to display ships left
		Image  m_ib_asteroids;						// banners
		Image  m_ib_star;
		Image  m_ib_lAsteroid;
		Image  m_ib_mAsteroid;
		Image  m_ib_sAsteroid;
		Image  m_ib_mine;
		Image  m_ib_enemy;
		Image  m_ib_bottom;

		size_t m_mineOne;							// sprite used for first spiked mine
		size_t m_mineTwo;							// sprite used for second spiked mine
		size_t m_ship;								// sprite used for player ship
		size_t m_bullet;							// sprite used for player bullet
		size_t m_cielingSprite;						// first sprite available for misc. asteroids

		Image* loadImages(string& imageName, size_t n, bool alpha = true);	// load a series of images into memory
		void   addShip();
		void   addAsteroid();
		void   addMedAsteroid(fMatrix location);
		void   addSmAsteroid(fMatrix location);
		void   addStar();
		void   addMine();
		void   addBullet();
		
		size_t takeASprite();
		void   returnASprite(size_t n);
		void   buildSpriteList();
		void   mouseLocation(int x, int y);
		void   shipMovement();
		void   sound(Sound s);
		void   gotStar(size_t n);
		void   shotStar(size_t n);
		void   shotMine(size_t source, size_t target);
		void   killShip();
		void   killBullet(size_t n);
		void   killAsteroid(size_t n);
		void   killStar(size_t n);
		void   killMine(size_t n);

		void   levelData();
		void   startLevel();
		void   gameOver();
		void   drawScore();
		void   drawBanners();
};

#endif // Asteroids_H